@@ -3,9 +3,9 @@
 #
 
 import logging
-from typing import List, Optional, Dict
+from typing import Dict, Iterator, List, Optional
 
-from volatility3.framework import constants, exceptions, renderers, interfaces, symbols
+from volatility3.framework import constants, exceptions, interfaces, renderers, symbols
 from volatility3.framework.configuration import requirements
 from volatility3.framework.objects import utility
 from volatility3.framework.renderers import format_hints
@@ -13,26 +13,14 @@ from volatility3.plugins.windows import pslist, psscan
 
 vollog = logging.getLogger(__name__)
 
-try:
-    import capstone
-
-    has_capstone = True
-except ImportError:
-    has_capstone = False
-
 
 class Handles(interfaces.plugins.PluginInterface):
     """Lists process open handles."""
 
     _required_framework_version = (2, 0, 0)
-    _version = (1, 0, 0)
+    _version = (4, 0, 0)
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._sar_value = None
-        self._type_map = None
-        self._cookie = None
-        self._level_mask = 7
+    LEVEL_MASK = 7
 
     @classmethod
     def get_requirements(cls) -> List[interfaces.configuration.RequirementInterface]:
@@ -43,11 +31,11 @@ class Handles(interfaces.plugins.PluginInterface):
                 description="Windows kernel",
                 architectures=["Intel32", "Intel64"],
             ),
-            requirements.PluginRequirement(
-                name="pslist", plugin=pslist.PsList, version=(2, 0, 0)
+            requirements.VersionRequirement(
+                name="pslist", component=pslist.PsList, version=(3, 0, 0)
             ),
             requirements.VersionRequirement(
-                name="psscan", component=psscan.PsScan, version=(1, 1, 0)
+                name="psscan", component=psscan.PsScan, version=(2, 0, 0)
             ),
             requirements.ListRequirement(
                 name="pid",
@@ -62,130 +50,83 @@ class Handles(interfaces.plugins.PluginInterface):
             ),
         ]
 
-    def _decode_pointer(self, value, magic):
-        """Windows encodes pointers to objects and decodes them on the fly
-        before using them.
-
-        This function mimics the decoding routine so we can generate the
-        proper pointer values as well.
+    @classmethod
+    def _get_item(
+        cls,
+        context: interfaces.context.ContextInterface,
+        kernel_module_name: str,
+        handle_table_entry: interfaces.objects.ObjectInterface,
+        handle_value: int,
+    ) -> Optional[interfaces.objects.ObjectInterface]:
+        """
+        Given  a handle table entry (_HANDLE_TABLE_ENTRY) structure from a
+        process' handle table, determine where the corresponding object's
+        _OBJECT_HEADER can be found, and construct and return the _OBJECT_HEADER
         """
 
-        value = value & 0xFFFFFFFFFFFFFFF8
-        value = value >> magic
-        # if (value & (1 << 47)):
-        #    value = value | 0xFFFF000000000000
-
-        return value
-
-    def _get_item(self, handle_table_entry, handle_value):
-        """Given  a handle table entry (_HANDLE_TABLE_ENTRY) structure from a
-        process' handle table, determine where the corresponding object's
-        _OBJECT_HEADER can be found."""
-
-        kernel = self.context.modules[self.config["kernel"]]
+        kernel = context.modules[kernel_module_name]
 
         virtual = kernel.layer_name
 
         try:
             # before windows 7
-            if not self.context.layers[virtual].is_valid(handle_table_entry.Object):
+            if not context.layers[virtual].is_valid(handle_table_entry.Object):
                 return None
             fast_ref = handle_table_entry.Object.cast("_EX_FAST_REF")
-            object_header = fast_ref.dereference().cast("_OBJECT_HEADER")
+
+            try:
+                object_header = fast_ref.dereference().cast("_OBJECT_HEADER")
+            except exceptions.InvalidAddressException:
+                return None
+
             object_header.GrantedAccess = handle_table_entry.GrantedAccess
         except AttributeError:
             # starting with windows 8
             is_64bit = symbols.symbol_table_is_64bit(
-                self.context, kernel.symbol_table_name
+                context=context, symbol_table_name=kernel.symbol_table_name
             )
 
             if is_64bit:
-                if handle_table_entry.LowValue == 0:
+                try:
+                    pointer_bits = handle_table_entry.ObjectPointerBits
+                except exceptions.InvalidAddressException:
                     return None
 
-                magic = self.find_sar_value()
+                if pointer_bits == 0:
+                    return None
 
-                # is this the right thing to raise here?
-                if magic is None:
-                    if has_capstone:
-                        raise AttributeError(
-                            "Unable to find the SAR value for decoding handle table pointers"
-                        )
-                    else:
-                        raise exceptions.MissingModuleException(
-                            "capstone",
-                            "Requires capstone to find the SAR value for decoding handle table pointers",
-                        )
+                offset = pointer_bits << 4
 
-                offset = self._decode_pointer(handle_table_entry.LowValue, magic)
             else:
-                if handle_table_entry.InfoTable == 0:
+                try:
+                    info_table = handle_table_entry.InfoTable
+                except exceptions.InvalidAddressException:
                     return None
 
-                offset = handle_table_entry.InfoTable & ~7
+                if info_table == 0:
+                    return None
+
+                offset = info_table & ~7
 
             # print("LowValue: {0:#x} Magic: {1:#x} Offset: {2:#x}".format(handle_table_entry.InfoTable, magic, offset))
-            object_header = self.context.object(
+            object_header = context.object(
                 kernel.symbol_table_name + constants.BANG + "_OBJECT_HEADER",
                 virtual,
                 offset=offset,
             )
-            object_header.GrantedAccess = handle_table_entry.GrantedAccessBits
+            try:
+                object_header.GrantedAccess = handle_table_entry.GrantedAccessBits
+            except exceptions.InvalidAddressException:
+                return None
 
         object_header.HandleValue = handle_value
         return object_header
 
-    def find_sar_value(self):
-        """Locate ObpCaptureHandleInformationEx if it exists in the sample.
-
-        Once found, parse it for the SAR value that we need to decode
-        pointers in the _HANDLE_TABLE_ENTRY which allows us to find the
-        associated _OBJECT_HEADER.
-        """
-
-        if self._sar_value is None:
-            if not has_capstone:
-                return None
-            kernel = self.context.modules[self.config["kernel"]]
-
-            virtual_layer_name = kernel.layer_name
-            kvo = self.context.layers[virtual_layer_name].config[
-                "kernel_virtual_offset"
-            ]
-            ntkrnlmp = self.context.module(
-                kernel.symbol_table_name, layer_name=virtual_layer_name, offset=kvo
-            )
-
-            try:
-                func_addr = ntkrnlmp.get_symbol("ObpCaptureHandleInformationEx").address
-            except exceptions.SymbolError:
-                return None
-
-            data = self.context.layers.read(virtual_layer_name, kvo + func_addr, 0x200)
-            if data is None:
-                return None
-
-            md = capstone.Cs(capstone.CS_ARCH_X86, capstone.CS_MODE_64)
-
-            for address, size, mnemonic, op_str in md.disasm_lite(
-                data, kvo + func_addr
-            ):
-                # print("{} {} {} {}".format(address, size, mnemonic, op_str))
-
-                if mnemonic.startswith("sar"):
-                    # if we don't want to parse op strings, we can disasm the
-                    # single sar instruction again, but we use disasm_lite for speed
-                    self._sar_value = int(op_str.split(",")[1].strip(), 16)
-                    break
-
-        return self._sar_value
-
     @classmethod
     def get_type_map(
         cls,
         context: interfaces.context.ContextInterface,
-        layer_name: str,
-        symbol_table: str,
+        kernel_module_name: str,
     ) -> Dict[int, str]:
         """List the executive object types (_OBJECT_TYPE) using the
         ObTypeIndexTable or ObpObjectTypes symbol (differs per OS). This method
@@ -207,17 +148,16 @@ class Handles(interfaces.plugins.PluginInterface):
 
         type_map: Dict[int, str] = {}
 
-        kvo = context.layers[layer_name].config["kernel_virtual_offset"]
-        ntkrnlmp = context.module(symbol_table, layer_name=layer_name, offset=kvo)
+        ntkrnlmp = context.modules[kernel_module_name]
 
         try:
             table_addr = ntkrnlmp.get_symbol("ObTypeIndexTable").address
         except exceptions.SymbolError:
             table_addr = ntkrnlmp.get_symbol("ObpObjectTypes").address
 
-        trans_layer = context.layers[layer_name]
+        trans_layer = context.layers[ntkrnlmp.layer_name]
 
-        if not trans_layer.is_valid(kvo + table_addr):
+        if not trans_layer.is_valid(ntkrnlmp.offset + table_addr):
             return type_map
 
         ptrs = ntkrnlmp.object(
@@ -235,13 +175,13 @@ class Handles(interfaces.plugins.PluginInterface):
 
             try:
                 objt = ptr.dereference().cast(
-                    symbol_table + constants.BANG + "_OBJECT_TYPE"
+                    ntkrnlmp.symbol_table_name + constants.BANG + "_OBJECT_TYPE"
                 )
                 type_name = objt.Name.String
             except exceptions.InvalidAddressException:
                 vollog.log(
                     constants.LOGLEVEL_VVV,
-                    f"Cannot access _OBJECT_HEADER Name at {objt.vol.offset:#x}",
+                    f"Cannot access _OBJECT_HEADER Name at {ptr.vol.offset:#x}",
                 )
                 continue
 
@@ -253,49 +193,52 @@ class Handles(interfaces.plugins.PluginInterface):
     def find_cookie(
         cls,
         context: interfaces.context.ContextInterface,
-        layer_name: str,
-        symbol_table: str,
+        kernel_module_name: str,
     ) -> Optional[interfaces.objects.ObjectInterface]:
         """Find the ObHeaderCookie value (if it exists)"""
 
+        kernel = context.modules[kernel_module_name]
+
         try:
-            offset = context.symbol_space.get_symbol(
-                symbol_table + constants.BANG + "ObHeaderCookie"
-            ).address
+            symbol_offset = kernel.get_symbol("ObHeaderCookie").address
         except exceptions.SymbolError:
+            vollog.debug('Unable to get symbol information for "ObHeaderCookie"')
             return None
 
-        kvo = context.layers[layer_name].config["kernel_virtual_offset"]
-        return context.object(
-            symbol_table + constants.BANG + "unsigned int",
-            layer_name,
-            offset=kvo + offset,
+        return kernel.object(
+            "unsigned int",
+            offset=symbol_offset,
         )
 
-    def _make_handle_array(self, offset, level, depth=0):
-        """Parse a process' handle table and yield valid handle table entries,
-        going as deep into the table "levels" as necessary."""
+    @classmethod
+    def _make_handle_array(
+        cls,
+        context: interfaces.context.ContextInterface,
+        kernel_module_name: str,
+        offset: int,
+        level: int,
+        depth: int = 0,
+    ) -> Iterator[interfaces.objects.ObjectInterface]:
+        """
+        Parses a process' handle table by constructing an array of
+        `_HANDLE_TABLE_ENTRY` structures at the given offset, and yields valid
+        handle table entries, going as deep into the table "levels" as
+        necessary.
+        """
 
-        kernel = self.context.modules[self.config["kernel"]]
-
-        virtual = kernel.layer_name
-        kvo = self.context.layers[virtual].config["kernel_virtual_offset"]
-
-        ntkrnlmp = self.context.module(
-            kernel.symbol_table_name, layer_name=virtual, offset=kvo
-        )
+        kernel = context.modules[kernel_module_name]
 
         if level > 0:
-            subtype = ntkrnlmp.get_type("pointer")
+            subtype = kernel.get_type("pointer")
             count = 0x1000 / subtype.size
         else:
-            subtype = ntkrnlmp.get_type("_HANDLE_TABLE_ENTRY")
+            subtype = kernel.get_type("_HANDLE_TABLE_ENTRY")
             count = 0x1000 / subtype.size
 
-        if not self.context.layers[virtual].is_valid(offset):
+        if not context.layers[kernel.layer_name].is_valid(offset):
             return None
 
-        table = ntkrnlmp.object(
+        table = kernel.object(
             object_type="array",
             offset=offset,
             subtype=subtype,
@@ -303,13 +246,27 @@ class Handles(interfaces.plugins.PluginInterface):
             absolute=True,
         )
 
-        layer_object = self.context.layers[virtual]
+        layer_object = context.layers[kernel.layer_name]
         masked_offset = offset & layer_object.maximum_address
 
-        for entry in table:
+        for i in range(len(table)):
+            try:
+                entry = table[i]
+            except exceptions.InvalidAddressException:
+                vollog.debug(f"Failed to get handle table entry at index {i}")
+                continue
+            # This triggered a backtrace in many testing samples
+            # in the level == 0 path
+            # The code above this calls `is_valid` on the `offset`
+            # It is sent but then does not validate `entry` before
+            # sending it to `_get_item`
+            if not context.layers[kernel.layer_name].is_valid(entry.vol.offset):
+                continue
+
             if level > 0:
-                for x in self._make_handle_array(entry, level - 1, depth):
-                    yield x
+                yield from cls._make_handle_array(
+                    context, kernel_module_name, entry, level - 1, depth
+                )
                 depth += 1
             else:
                 handle_multiplier = 4
@@ -320,7 +277,7 @@ class Handles(interfaces.plugins.PluginInterface):
                     / (subtype.size / handle_multiplier)
                 ) + handle_level_base
 
-                item = self._get_item(entry, handle_value)
+                item = cls._get_item(context, kernel_module_name, entry, handle_value)
 
                 if item is None:
                     continue
@@ -334,10 +291,21 @@ class Handles(interfaces.plugins.PluginInterface):
                 except exceptions.InvalidAddressException:
                     continue
 
-    def handles(self, handle_table):
+    @classmethod
+    def handles(
+        cls,
+        context: interfaces.context.ContextInterface,
+        kernel_module_name: str,
+        handle_table: interfaces.objects.ObjectInterface,
+    ) -> Iterator[interfaces.objects.ObjectInterface]:
+        """
+        Takes a context, kernel module name, and handle table structure
+        (_HANDLE_TABLE), and yields _HANDLE_TABLE_ENTRY structures from the
+        handle table.
+        """
         try:
-            TableCode = handle_table.TableCode & ~self._level_mask
-            table_levels = handle_table.TableCode & self._level_mask
+            TableCode = handle_table.TableCode & ~cls.LEVEL_MASK
+            table_levels = handle_table.TableCode & cls.LEVEL_MASK
         except exceptions.InvalidAddressException:
             vollog.log(
                 constants.LOGLEVEL_VVV,
@@ -345,22 +313,17 @@ class Handles(interfaces.plugins.PluginInterface):
             )
             return None
 
-        for handle_table_entry in self._make_handle_array(TableCode, table_levels):
-            yield handle_table_entry
+        yield from cls._make_handle_array(
+            context, kernel_module_name, TableCode, table_levels
+        )
 
     def _generator(self, procs):
-        kernel = self.context.modules[self.config["kernel"]]
-
         type_map = self.get_type_map(
-            context=self.context,
-            layer_name=kernel.layer_name,
-            symbol_table=kernel.symbol_table_name,
+            context=self.context, kernel_module_name=self.config["kernel"]
         )
 
         cookie = self.find_cookie(
-            context=self.context,
-            layer_name=kernel.layer_name,
-            symbol_table=kernel.symbol_table_name,
+            context=self.context, kernel_module_name=self.config["kernel"]
         )
 
         for proc in procs:
@@ -375,7 +338,9 @@ class Handles(interfaces.plugins.PluginInterface):
 
             process_name = utility.array_to_string(proc.ImageFileName)
 
-            for entry in self.handles(object_table):
+            for entry in self.handles(
+                self.context, self.config["kernel"], object_table
+            ):
                 try:
                     obj_type = entry.get_object_type(type_map, cookie)
                     if obj_type is None:
@@ -398,7 +363,7 @@ class Handles(interfaces.plugins.PluginInterface):
                         try:
                             obj_name = entry.NameInfo.Name.String
                         except (ValueError, exceptions.InvalidAddressException):
-                            obj_name = ""
+                            obj_name = None
 
                 except exceptions.InvalidAddressException:
                     vollog.log(
@@ -416,7 +381,7 @@ class Handles(interfaces.plugins.PluginInterface):
                         format_hints.Hex(entry.HandleValue),
                         obj_type,
                         format_hints.Hex(entry.GrantedAccess),
-                        obj_name,
+                        obj_name or renderers.NotAvailableValue(),
                     ),
                 )
 
@@ -427,8 +392,7 @@ class Handles(interfaces.plugins.PluginInterface):
         if self.config["offset"]:
             procs = psscan.PsScan.scan_processes(
                 self.context,
-                kernel.layer_name,
-                kernel.symbol_table_name,
+                self.config["kernel"],
                 filter_func=psscan.PsScan.create_offset_filter(
                     self.context,
                     kernel.layer_name,
@@ -438,8 +402,7 @@ class Handles(interfaces.plugins.PluginInterface):
         else:
             procs = pslist.PsList.list_processes(
                 context=self.context,
-                layer_name=kernel.layer_name,
-                symbol_table=kernel.symbol_table_name,
+                kernel_module_name=self.config["kernel"],
                 filter_func=filter_func,
             )
 
